from django.contrib import admin
from flag.models import FlaggedContent, FlagInstance

class InlineFlagInstance(admin.TabularInline):
    model = FlagInstance
    extra = 0

class FlaggedContentAdmin(admin.ModelAdmin):
    inlines = [InlineFlagInstance]

admin.site.register(FlaggedContent, FlaggedContentAdmin)



from django.http import HttpResponseRedirect, Http404
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404

from flag.models import add_flag

@login_required
def flag(request):
    
    content_type = request.POST.get("content_type")
    object_id = request.POST.get("object_id")
    creator_field = request.POST.get("creator_field")
    comment = request.POST.get("comment")
    next = request.POST.get("next")
    
    content_type = get_object_or_404(ContentType, id = int(content_type))
    object_id = int(object_id)
    
    content_object = content_type.get_object_for_this_type(id=object_id)
    
    if creator_field and hasattr(content_object, creator_field):
        creator = getattr(content_object, creator_field)
    else:
        creator = None
    
    add_flag(request.user, content_type, object_id, creator, comment)
    request.user.message_set.create(message="You have added a flag. A moderator will review your submission shortly.")
    
    if next:
        return HttpResponseRedirect(next)
    else:
        return Http404



from datetime import datetime

from django.db import models

from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

from django.contrib.auth.models import User


STATUS = (
    ('1', 'flagged'),
    ('2', 'flag rejected by moderator'),
    ('3', 'creator notified'),
    ('4', 'content removed by creator'),
    ('5', 'content removed by moderator'),
)

class FlaggedContent(models.Model):
    
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    creator = models.ForeignKey(User, related_name="flagged_content") # user who created flagged content -- this is kept in model so it outlives content
    status = models.CharField(max_length=1, choices=STATUS, default='1')    
    moderator = models.ForeignKey(User, null=True, related_name="moderated_content") # moderator responsible for last status change


class FlagInstance(models.Model):
    
    flagged_content = models.ForeignKey(FlaggedContent)
    user = models.ForeignKey(User) # user flagging the content
    when_added = models.DateTimeField(default=datetime.now)
    when_recalled = models.DateTimeField(null=True) # if recalled at all
    comment = models.TextField() # comment by the flagger


def add_flag(flagger, content_type, object_id, content_creator, comment):
    
    # check if it's already been flagged
    try:
        flagged_content = FlaggedContent.objects.get(content_type=content_type, object_id=object_id)
    except FlaggedContent.DoesNotExist:
        flagged_content = FlaggedContent(content_type=content_type, object_id=object_id, creator=content_creator)
        flagged_content.save()
    
    flag_instance = FlagInstance(flagged_content=flagged_content, user=flagger, comment=comment)
    flag_instance.save()
    
    return flag_instance



from setuptools import setup, find_packages
 
setup(
    name='django-flag',
    version='0.1.1',
    description='flagging of inapproriate/spam content',
    author='Greg Newman',
    author_email='greg@20seven.org',
    url='http://code.google.com/p/django-flag/',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    include_package_data=True,
    zip_safe=False,
)



from django import template

from django.contrib.contenttypes.models import ContentType

register = template.Library()

def flag(context, content_object, creator_field):
    content_type = ContentType.objects.get(app_label=content_object._meta.app_label, model=content_object._meta.module_name)
    return {
        'content_type': content_type.id,
        'object_id': content_object.id,
        'creator_field': creator_field,
        'request': context['request'],
        'user': context['user'],
    }


register.inclusion_tag('flag/flag_form.html', takes_context=True)(flag)
